from __future__ import annotations

from typing import List, Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.common import APIResponse
from ...types.scenario import (
    Scenario,
    ScenarioPreview,
    ScenarioCreateParams,
    ScenarioUpdateParams,
    ScenarioPreviewParams,
)

__all__ = ["ScenariosResource", "AsyncScenariosResource"]


class ScenariosResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ScenariosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return ScenariosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScenariosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return ScenariosResourceWithStreamingResponse(self)

    def create(
        self,
        project_id: str,
        *,
        name: str,
        description: str,
        rules: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scenario:
        """Create a new scenario within a project.

        Parameters
        ----------
        project_id : str
            The ID of the project.
        name : str
            The name of the scenario.
        description : str
            The description of the scenario.
        rules : SequenceNotStr[str]
            The rules of the scenario.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scenario
            The newly created scenario.

        Raises
        ------
        ValueError
            If ``project_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        response = self._post(
            f"/v2/projects/{project_id}/scenarios",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "rules": rules,
                },
                ScenarioCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scenario],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        scenario_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scenario:
        """Retrieve a scenario by its ID within a project.

        Parameters
        ----------
        scenario_id : str
            The ID of the scenario.
        project_id : str
            The ID of the project.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scenario
            The requested scenario.

        Raises
        ------
        ValueError
            If ``project_id`` or ``scenario_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not scenario_id:
            raise ValueError(f"Expected a non-empty value for `scenario_id` but received {scenario_id!r}")
        response = self._get(
            f"/v2/projects/{project_id}/scenarios/{scenario_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scenario],
        )

        return self._unwrap(response)

    def update(
        self,
        scenario_id: str,
        *,
        project_id: str,
        name: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        rules: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scenario:
        """Update an existing scenario's definition.

        Parameters
        ----------
        scenario_id : str
            The ID of the scenario.
        project_id : str
            The ID of the project.
        name : str or None
            Name of the scenario.
        description : str or None
            Description of the scenario.
        rules : SequenceNotStr[str] or None
            The rules of the scenario.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scenario
            The updated scenario.

        Raises
        ------
        ValueError
            If ``project_id`` or ``scenario_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not scenario_id:
            raise ValueError(f"Expected a non-empty value for `scenario_id` but received {scenario_id!r}")
        response = self._patch(
            f"/v2/projects/{project_id}/scenarios/{scenario_id}",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "rules": rules,
                },
                ScenarioUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scenario],
        )

        return self._unwrap(response)

    def list(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Scenario]:
        """List all scenarios for a project.

        Parameters
        ----------
        project_id : str
            The ID of the project.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of Scenario
            A list of all scenarios for the project.

        Raises
        ------
        ValueError
            If ``project_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        response = self._get(
            f"/v2/projects/{project_id}/scenarios",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[Scenario]],
        )

        return self._unwrap(response)

    def delete(
        self,
        scenario_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a scenario from a project.

        Parameters
        ----------
        scenario_id : str
            The ID of the scenario.
        project_id : str
            The ID of the project.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``project_id`` or ``scenario_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not scenario_id:
            raise ValueError(f"Expected a non-empty value for `scenario_id` but received {scenario_id!r}")
        response = self._delete(
            f"/v2/projects/{project_id}/scenarios/{scenario_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def preview(
        self,
        project_id: str,
        *,
        agent_id: Optional[str] | Omit = omit,
        description: str,
        rules: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScenarioPreview:
        """Generate a preview conversation for a scenario without persisting it.

        Parameters
        ----------
        project_id : str
            The ID of the project.
        agent_id : str or None
            Agent ID to use for preview.
        description : str
            Description of the scenario.
        rules : SequenceNotStr[str]
            Rules to use for preview.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScenarioPreview
            The generated preview conversation.

        Raises
        ------
        ValueError
            If ``project_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        response = self._post(
            f"/v2/projects/{project_id}/scenarios/preview",
            body=maybe_transform(
                {
                    "description": description,
                    "rules": rules,
                },
                ScenarioPreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"agent_id": agent_id}, ScenarioPreviewParams),
            ),
            cast_to=APIResponse[ScenarioPreview],
        )

        return self._unwrap(response)


class AsyncScenariosResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncScenariosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScenariosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScenariosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncScenariosResourceWithStreamingResponse(self)

    async def create(
        self,
        project_id: str,
        *,
        name: str,
        description: str,
        rules: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scenario:
        """Create a new scenario within a project.

        Parameters
        ----------
        project_id : str
            The ID of the project.
        name : str
            The name of the scenario.
        description : str
            The description of the scenario.
        rules : SequenceNotStr[str]
            The rules of the scenario.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scenario
            The newly created scenario.

        Raises
        ------
        ValueError
            If ``project_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        response = await self._post(
            f"/v2/projects/{project_id}/scenarios",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "rules": rules,
                },
                ScenarioCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scenario],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        scenario_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scenario:
        """Retrieve a scenario by its ID within a project.

        Parameters
        ----------
        scenario_id : str
            The ID of the scenario.
        project_id : str
            The ID of the project.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scenario
            The requested scenario.

        Raises
        ------
        ValueError
            If ``project_id`` or ``scenario_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not scenario_id:
            raise ValueError(f"Expected a non-empty value for `scenario_id` but received {scenario_id!r}")
        response = await self._get(
            f"/v2/projects/{project_id}/scenarios/{scenario_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scenario],
        )

        return self._unwrap(response)

    async def update(
        self,
        scenario_id: str,
        *,
        project_id: str,
        name: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        rules: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Scenario:
        """Update an existing scenario's definition.

        Parameters
        ----------
        scenario_id : str
            The ID of the scenario.
        project_id : str
            The ID of the project.
        name : str or None
            Name of the scenario.
        description : str or None
            Description of the scenario.
        rules : SequenceNotStr[str] or None
            The rules of the scenario.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Scenario
            The updated scenario.

        Raises
        ------
        ValueError
            If ``project_id`` or ``scenario_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not scenario_id:
            raise ValueError(f"Expected a non-empty value for `scenario_id` but received {scenario_id!r}")
        response = await self._patch(
            f"/v2/projects/{project_id}/scenarios/{scenario_id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "rules": rules,
                },
                ScenarioUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Scenario],
        )

        return self._unwrap(response)

    async def list(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Scenario]:
        """List all scenarios for a project.

        Parameters
        ----------
        project_id : str
            The ID of the project.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        list of Scenario
            A list of all scenarios for the project.

        Raises
        ------
        ValueError
            If ``project_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        response = await self._get(
            f"/v2/projects/{project_id}/scenarios",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[Scenario]],
        )

        return self._unwrap(response)

    async def delete(
        self,
        scenario_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a scenario from a project.

        Parameters
        ----------
        scenario_id : str
            The ID of the scenario.
        project_id : str
            The ID of the project.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``project_id`` or ``scenario_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not scenario_id:
            raise ValueError(f"Expected a non-empty value for `scenario_id` but received {scenario_id!r}")
        response = await self._delete(
            f"/v2/projects/{project_id}/scenarios/{scenario_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def preview(
        self,
        project_id: str,
        *,
        agent_id: Optional[str] | Omit = omit,
        description: str,
        rules: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScenarioPreview:
        """Generate a preview conversation for a scenario without persisting it.

        Parameters
        ----------
        project_id : str
            The ID of the project.
        agent_id : str or None
            Agent ID to use for preview.
        description : str
            Description of the scenario.
        rules : SequenceNotStr[str]
            Rules to use for preview.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float or httpx.Timeout or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScenarioPreview
            The generated preview conversation.

        Raises
        ------
        ValueError
            If ``project_id`` is empty.
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        response = await self._post(
            f"/v2/projects/{project_id}/scenarios/preview",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "rules": rules,
                },
                ScenarioPreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"agent_id": agent_id}, ScenarioPreviewParams),
            ),
            cast_to=APIResponse[ScenarioPreview],
        )

        return self._unwrap(response)


class ScenariosResourceWithRawResponse:
    def __init__(self, scenarios: ScenariosResource) -> None:
        self._scenarios = scenarios

        self.create = to_raw_response_wrapper(
            scenarios.create,
        )
        self.retrieve = to_raw_response_wrapper(
            scenarios.retrieve,
        )
        self.update = to_raw_response_wrapper(
            scenarios.update,
        )
        self.list = to_raw_response_wrapper(
            scenarios.list,
        )
        self.delete = to_raw_response_wrapper(
            scenarios.delete,
        )
        self.preview = to_raw_response_wrapper(
            scenarios.preview,
        )


class AsyncScenariosResourceWithRawResponse:
    def __init__(self, scenarios: AsyncScenariosResource) -> None:
        self._scenarios = scenarios

        self.create = async_to_raw_response_wrapper(
            scenarios.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            scenarios.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            scenarios.update,
        )
        self.list = async_to_raw_response_wrapper(
            scenarios.list,
        )
        self.delete = async_to_raw_response_wrapper(
            scenarios.delete,
        )
        self.preview = async_to_raw_response_wrapper(
            scenarios.preview,
        )


class ScenariosResourceWithStreamingResponse:
    def __init__(self, scenarios: ScenariosResource) -> None:
        self._scenarios = scenarios

        self.create = to_streamed_response_wrapper(
            scenarios.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            scenarios.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            scenarios.update,
        )
        self.list = to_streamed_response_wrapper(
            scenarios.list,
        )
        self.delete = to_streamed_response_wrapper(
            scenarios.delete,
        )
        self.preview = to_streamed_response_wrapper(
            scenarios.preview,
        )


class AsyncScenariosResourceWithStreamingResponse:
    def __init__(self, scenarios: AsyncScenariosResource) -> None:
        self._scenarios = scenarios

        self.create = async_to_streamed_response_wrapper(
            scenarios.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            scenarios.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            scenarios.update,
        )
        self.list = async_to_streamed_response_wrapper(
            scenarios.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            scenarios.delete,
        )
        self.preview = async_to_streamed_response_wrapper(
            scenarios.preview,
        )



"""Knowledge base domain types."""

from typing import Dict, List, Literal, Optional, TypedDict
from datetime import datetime
from typing_extensions import Required

from pydantic import computed_field

from .common import TaskState, OrderByParam, TaskProgress, FilterValueParam, TaskProgressParam
from .._types import FileTypes, SequenceNotStr
from .._models import BaseModel

__all__ = [
    "KnowledgeBase",
    "KnowledgeBaseReference",
    "Topic",
    "KnowledgeBaseDocumentRow",
    "KnowledgeBaseDocumentDetail",
    "KnowledgeBaseListParams",
    "KnowledgeBaseCreateParams",
    "KnowledgeBaseUpdateParams",
    "KnowledgeBaseBulkDeleteParams",
    "KnowledgeBaseSearchDocumentsParams",
    "KnowledgeBaseDocumentOrderByParam",
    "KnowledgeBaseDocumentFiltersParam",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class Topic(BaseModel):
    id: str
    created_at: datetime
    knowledge_base_id: str
    name: str
    updated_at: datetime
    document_count: Optional[int] = None


class KnowledgeBaseReference(BaseModel):
    id: str
    name: str


class KnowledgeBase(BaseModel):
    id: str
    created_at: datetime
    description: Optional[str] = None
    filename: Optional[str] = None
    n_documents: int
    name: str
    project_id: str
    status: TaskProgress
    topics: List[Topic]
    updated_at: datetime

    @property
    def state(self) -> TaskState:
        return self.status.state


class KnowledgeBaseDocumentRow(BaseModel):
    id: str
    created_at: datetime
    knowledge_base_id: str
    snippet: str
    updated_at: datetime
    topic_id: Optional[str] = None
    topic_name: Optional[str] = None

    @computed_field
    def content(self) -> str:
        return self.snippet


class KnowledgeBaseDocumentDetail(BaseModel):
    id: str
    content: str
    created_at: datetime
    knowledge_base_id: str
    updated_at: datetime
    topic_id: Optional[str] = None
    topic_name: Optional[str] = None


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class KnowledgeBaseListParams(TypedDict, total=False):
    project_id: Optional[str]


class KnowledgeBaseCreateParams(TypedDict, total=False):
    project_id: Required[str]
    name: Required[str]
    description: Optional[str]
    document_column: str
    topic_column: str
    file: Required[FileTypes]


class KnowledgeBaseUpdateParams(TypedDict, total=False):
    description: Optional[str]
    name: Optional[str]
    project_id: Optional[str]
    status: Optional[TaskProgressParam]


class KnowledgeBaseBulkDeleteParams(TypedDict, total=False):
    knowledge_base_ids: Required[SequenceNotStr[str]]


# ---------------------------------------------------------------------------
# Search params
# ---------------------------------------------------------------------------

KnowledgeBaseDocumentSortColumn = Literal["created_at", "updated_at", "topic_id"]
KnowledgeBaseDocumentFilterColumn = Literal["topic_id"]

KnowledgeBaseDocumentOrderByParam = OrderByParam[KnowledgeBaseDocumentSortColumn]
KnowledgeBaseDocumentFiltersParam = Dict[KnowledgeBaseDocumentFilterColumn, FilterValueParam]


class KnowledgeBaseSearchDocumentsParams(TypedDict, total=False):
    search: Optional[str]
    order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]]
    filters: Optional[KnowledgeBaseDocumentFiltersParam]
    limit: int
    offset: int



from typing import Any, List, Iterable

import pytest

from giskard_hub.types.chat import ChatMessage
from giskard_hub.types.common import TaskState
from giskard_hub.types.test_case import TestCase
from giskard_hub.types.evaluation import Evaluation
from giskard_hub.resources.helpers import HelpersResource, AsyncHelpersResource


class DummyStateful:
    def __init__(self, id: str, state: TaskState) -> None:
        self.id = id
        self._state: TaskState = state

    @property
    def state(self) -> TaskState:
        return self._state


class DummySyncResource:
    def __init__(self, states: Iterable[TaskState]) -> None:
        self._states: List[TaskState] = list(states)
        self._index = 0

    def retrieve(self, _id: str) -> DummyStateful:
        # Return successive states until we run out, then repeat last state
        if self._index < len(self._states) - 1:
            state = self._states[self._index]
            self._index += 1
        else:
            state = self._states[-1]
        return DummyStateful(_id, state)


class DummyAsyncResource:
    def __init__(self, states: Iterable[TaskState]) -> None:
        self._states: List[TaskState] = list(states)
        self._index = 0

    async def retrieve(self, _id: str) -> DummyStateful:
        if self._index < len(self._states) - 1:
            state = self._states[self._index]
            self._index += 1
        else:
            state = self._states[-1]
        return DummyStateful(_id, state)


class DummySyncHelpers(HelpersResource):
    def __init__(self, resource: DummySyncResource) -> None:
        # bypass parent init; we don't need a real client
        self._resource = resource

    def _map_entity_to_resource(self, _: DummyStateful) -> DummySyncResource:  # type: ignore[override]
        return self._resource


class DummyAsyncHelpers(AsyncHelpersResource):
    def __init__(self, resource: DummyAsyncResource) -> None:
        # bypass parent init; we don't need a real client
        self._resource = resource

    def _map_entity_to_resource(self, _: DummyStateful) -> DummyAsyncResource:  # type: ignore[override]
        return self._resource


def test_wait_for_completion_sync_success() -> None:
    helper = DummySyncHelpers(DummySyncResource(["running", "running", "finished"]))
    entity = DummyStateful("id-1", "running")

    result = helper.wait_for_completion(
        entity,
        poll_interval=0,
        max_retries=5,
        running_states={"running"},
        error_states={"error"},
    )

    assert isinstance(result, DummyStateful)
    assert result.state == "finished"  # type: ignore[comparison-overlap]


def test_wait_for_completion_sync_error_state_raises() -> None:
    helper = DummySyncHelpers(DummySyncResource(["running", "error"]))
    entity = DummyStateful("id-1", "running")

    with pytest.raises(ValueError):
        helper.wait_for_completion(
            entity,
            poll_interval=0,
            max_retries=3,
            running_states={"running"},
            error_states={"error"},
            raise_on_error=True,
        )


@pytest.mark.asyncio
async def test_wait_for_completion_async_success() -> None:
    helper = DummyAsyncHelpers(DummyAsyncResource(["running", "finished"]))
    entity = DummyStateful("id-1", "running")

    result = await helper.wait_for_completion(
        entity,
        poll_interval=0,
        max_retries=5,
        running_states={"running"},
        error_states={"error"},
    )

    assert isinstance(result, DummyStateful)
    assert result.state == "finished"  # type: ignore[comparison-overlap]


@pytest.mark.asyncio
async def test_wait_for_completion_async_error_state_raises() -> None:
    helper = DummyAsyncHelpers(DummyAsyncResource(["running", "error"]))
    entity = DummyStateful("id-1", "running")

    with pytest.raises(ValueError):
        await helper.wait_for_completion(
            entity,
            poll_interval=0,
            max_retries=3,
            running_states={"running"},
            error_states={"error"},
            raise_on_error=True,
        )


def test_wait_for_completion_sync_timeout() -> None:
    states: List[TaskState] = ["running"] * 3
    helper = DummySyncHelpers(DummySyncResource(states))
    entity = DummyStateful("id-1", "running")

    with pytest.raises(RuntimeError):
        helper.wait_for_completion(
            entity,
            poll_interval=0,
            max_retries=2,
        )


def test_wait_for_completion_sync_error_no_raise() -> None:
    helper = DummySyncHelpers(DummySyncResource(["running", "error"]))
    entity = DummyStateful("id-1", "running")

    result = helper.wait_for_completion(
        entity,
        poll_interval=0,
        max_retries=3,
        error_states={"error"},
        raise_on_error=False,
    )
    assert result.state == "error"


# ---------------------------------------------------------------------------
# evaluate helpers (unit-style tests with fake clients)
# ---------------------------------------------------------------------------


class DummyEvaluations:
    def __init__(self, remote_eval: Evaluation, local_eval: Evaluation, entries: list[Any]) -> None:
        self.remote_eval = remote_eval
        self.local_eval = local_eval
        self.entries = entries
        self.create_kwargs: dict[str, Any] | None = None
        self.create_local_kwargs: dict[str, Any] | None = None

        class _Results:
            def __init__(self, outer: "DummyEvaluations") -> None:
                self._outer = outer
                self.submissions: list[dict[str, Any]] = []

            def list(self, *, evaluation_id: str, include: list[str]) -> list[Any]:
                assert "test_case" in include
                assert evaluation_id == outer.local_eval.id  # type: ignore[union-attr]
                return outer.entries

            def submit_local_output(self, *, result_id: str, evaluation_id: str, agent_output: dict[str, Any]) -> None:
                self.submissions.append(
                    {
                        "result_id": result_id,
                        "evaluation_id": evaluation_id,
                        "agent_output": agent_output,
                    }
                )

        outer = self
        self.results = _Results(outer)

    def create(self, **kwargs: Any) -> Evaluation:
        self.create_kwargs = kwargs
        return self.remote_eval

    def create_local(self, **kwargs: Any) -> Evaluation:
        self.create_local_kwargs = kwargs
        return self.local_eval


class DummyClient:
    def __init__(self, evaluations: DummyEvaluations) -> None:
        self.evaluations = evaluations


class HelpersWithDummyClient(HelpersResource):
    _client: Any

    def __init__(self, client: DummyClient) -> None:  # type: ignore[override]
        # Bypass parent initialisation; only `_client` is used in `evaluate`.
        object.__setattr__(self, "_client", client)


def _make_evaluation(eval_id: str = "eval-1") -> Evaluation:
    # Use minimal construction; other fields aren't inspected in these tests.
    return Evaluation.model_construct(id=eval_id)  # type: ignore[arg-type]


def _make_test_case(messages: list[ChatMessage]) -> TestCase:
    return TestCase.model_construct(id="tc-1", messages=messages)  # type: ignore[arg-type]


def _make_result_entry(test_case: TestCase, result_id: str = "res-1") -> Any:
    class Entry:
        def __init__(self, id: str, test_case: TestCase) -> None:
            self.id = id
            self.test_case = test_case

    return Entry(result_id, test_case)


def test_evaluate_remote_with_ids() -> None:
    remote_eval = _make_evaluation("remote-eval")
    local_eval = _make_evaluation("local-eval")
    evaluations = DummyEvaluations(remote_eval, local_eval, entries=[])
    client = DummyClient(evaluations)
    helpers = HelpersWithDummyClient(client)  # type: ignore[arg-type]

    result = helpers.evaluate(
        agent="agent-1",
        dataset="ds-1",
        project="proj-1",
        name="my-eval",
        tags=["tag-1"],
    )

    assert result is remote_eval
    assert evaluations.create_kwargs is not None
    assert evaluations.create_kwargs["project_id"] == "proj-1"
    assert evaluations.create_kwargs["agent_id"] == "agent-1"
    assert evaluations.create_kwargs["name"] == "my-eval"
    assert evaluations.create_kwargs["dataset_id"] == "ds-1"
    assert evaluations.create_kwargs["tags"] == ["tag-1"]


def test_evaluate_remote_missing_project_raises() -> None:
    remote_eval = _make_evaluation()
    local_eval = _make_evaluation()
    evaluations = DummyEvaluations(remote_eval, local_eval, entries=[])
    client = DummyClient(evaluations)
    helpers = HelpersWithDummyClient(client)  # type: ignore[arg-type]

    with pytest.raises(ValueError, match="Project is required"):
        helpers.evaluate(
            agent="agent-1",
            dataset="ds-1",
            project=None,
        )


def test_evaluate_local_with_callable_str_output() -> None:
    remote_eval = _make_evaluation()
    local_eval = _make_evaluation("local-eval")

    messages = [ChatMessage(role="user", content="hi")]
    test_case = _make_test_case(messages)
    entry = _make_result_entry(test_case, "res-1")

    evaluations = DummyEvaluations(remote_eval, local_eval, entries=[entry])
    client = DummyClient(evaluations)
    helpers = HelpersWithDummyClient(client)  # type: ignore[arg-type]

    def agent_fn(msgs: list[ChatMessage]) -> str:
        assert msgs == messages
        return "hello"

    result = helpers.evaluate(
        agent=agent_fn,
        dataset="ds-1",
        name="local-eval",
        tags=["tag-1"],
    )

    assert result is local_eval
    assert evaluations.create_local_kwargs is not None
    assert evaluations.create_local_kwargs["agent_info"]["name"] == agent_fn.__name__

    submissions = evaluations.results.submissions
    assert len(submissions) == 1
    submitted = submissions[0]["agent_output"]
    assert submitted["response"]["content"] == "hello"
    assert submitted["response"]["role"] == "assistant"


def test_evaluate_local_with_callable_agent_output_dict() -> None:
    remote_eval = _make_evaluation()
    local_eval = _make_evaluation("local-eval")

    messages = [ChatMessage(role="user", content="hi")]
    test_case = _make_test_case(messages)
    entry = _make_result_entry(test_case, "res-1")

    evaluations = DummyEvaluations(remote_eval, local_eval, entries=[entry])
    client = DummyClient(evaluations)
    helpers = HelpersWithDummyClient(client)  # type: ignore[arg-type]

    def agent_fn(msgs: list[ChatMessage]) -> dict[str, Any]:
        assert msgs == messages
        # Dict with AgentOutput-like shape; should be accepted by the helper.
        return {
            "response": {
                "role": "assistant",
                "content": "dict response",
            }
        }

    result = helpers.evaluate(
        agent=agent_fn,  # type: ignore[arg-type]
        dataset="ds-1",
    )

    assert result is local_eval
    submissions = evaluations.results.submissions
    assert len(submissions) == 1
    submitted = submissions[0]["agent_output"]
    assert submitted["response"]["content"] == "dict response"
    assert submitted["response"]["role"] == "assistant"



from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.scan import Severity, ReviewStatus, ScanProbeAttempt, ScanProbeAttemptUpdateParams
from ..._base_client import make_request_options
from ...types.common import APIResponse

__all__ = ["AttemptsResource", "AsyncAttemptsResource"]


class AttemptsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AttemptsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AttemptsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AttemptsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AttemptsResourceWithStreamingResponse(self)

    def update(
        self,
        probe_attempt_id: str,
        *,
        review_status: Optional[ReviewStatus] | Omit = omit,
        severity: Optional[Severity] | Omit = omit,
        successful: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScanProbeAttempt:
        """Update a scan probe attempt's review status, severity, or success flag.

        Parameters
        ----------
        probe_attempt_id : str
            Scan Probe Attempt ID to update.
        review_status : ReviewStatus or None
            Review status of the attempt.
        severity : Severity or None
            Severity of the attempt.
        successful : bool or None
            Whether the attempt was successful.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScanProbeAttempt
            The updated scan probe attempt object.

        Raises
        ------
        ValueError
            If ``probe_attempt_id`` is empty.
        """
        if not probe_attempt_id:
            raise ValueError(f"Expected a non-empty value for `probe_attempt_id` but received {probe_attempt_id!r}")
        response = self._patch(
            f"/v2/scan-attempts/{probe_attempt_id}",
            body=maybe_transform(
                {
                    "review_status": review_status,
                    "severity": severity,
                    "successful": successful,
                },
                ScanProbeAttemptUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScanProbeAttempt],
        )

        return self._unwrap(response)


class AsyncAttemptsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAttemptsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAttemptsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAttemptsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncAttemptsResourceWithStreamingResponse(self)

    async def update(
        self,
        probe_attempt_id: str,
        *,
        review_status: Optional[ReviewStatus] | Omit = omit,
        severity: Optional[Severity] | Omit = omit,
        successful: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScanProbeAttempt:
        """Update a scan probe attempt's review status, severity, or success flag.

        Parameters
        ----------
        probe_attempt_id : str
            Scan Probe Attempt ID to update.
        review_status : ReviewStatus or None
            Review status of the attempt.
        severity : Severity or None
            Severity of the attempt.
        successful : bool or None
            Whether the attempt was successful.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, or None
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        ScanProbeAttempt
            The updated scan probe attempt object.

        Raises
        ------
        ValueError
            If ``probe_attempt_id`` is empty.
        """
        if not probe_attempt_id:
            raise ValueError(f"Expected a non-empty value for `probe_attempt_id` but received {probe_attempt_id!r}")
        response = await self._patch(
            f"/v2/scan-attempts/{probe_attempt_id}",
            body=await async_maybe_transform(
                {
                    "review_status": review_status,
                    "severity": severity,
                    "successful": successful,
                },
                ScanProbeAttemptUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[ScanProbeAttempt],
        )

        return self._unwrap(response)


class AttemptsResourceWithRawResponse:
    def __init__(self, attempts: AttemptsResource) -> None:
        self._attempts = attempts

        self.update = to_raw_response_wrapper(
            attempts.update,
        )


class AsyncAttemptsResourceWithRawResponse:
    def __init__(self, attempts: AsyncAttemptsResource) -> None:
        self._attempts = attempts

        self.update = async_to_raw_response_wrapper(
            attempts.update,
        )


class AttemptsResourceWithStreamingResponse:
    def __init__(self, attempts: AttemptsResource) -> None:
        self._attempts = attempts

        self.update = to_streamed_response_wrapper(
            attempts.update,
        )


class AsyncAttemptsResourceWithStreamingResponse:
    def __init__(self, attempts: AsyncAttemptsResource) -> None:
        self._attempts = attempts

        self.update = async_to_streamed_response_wrapper(
            attempts.update,
        )



from __future__ import annotations

import json
from typing import Any, List, Tuple, Literal, Mapping, Optional, cast, overload
from pathlib import Path

import httpx

from ..types import (
    APIPaginatedResponse,
    KnowledgeBaseDocumentRow,
    KnowledgeBaseDocumentFiltersParam,
    KnowledgeBaseDocumentOrderByParam,
    KnowledgeBaseSearchDocumentsParams,
)
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, SequenceNotStr, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.common import APIResponse, TaskProgressParam, APIPaginatedMetadata
from ..types.knowledge_base import (
    KnowledgeBase,
    KnowledgeBaseListParams,
    KnowledgeBaseCreateParams,
    KnowledgeBaseUpdateParams,
    KnowledgeBaseDocumentDetail,
    KnowledgeBaseBulkDeleteParams,
)

__all__ = ["KnowledgeBasesResource", "AsyncKnowledgeBasesResource"]


class KnowledgeBasesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> KnowledgeBasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return KnowledgeBasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KnowledgeBasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return KnowledgeBasesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        project_id: str,
        data: FileTypes | list[dict[str, Any]] | str,
        description: Optional[str] | Omit = omit,
        document_column: str | Omit = omit,
        topic_column: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBase:
        """Create a new knowledge base by uploading a file of documents.

        Parameters
        ----------
        name : str
            Name of the knowledge base.
        project_id : str
            Project ID to create the knowledge base in.
        data : FileTypes | list[dict[str, Any]] | str
            Data to upload for the knowledge base.
        description : Optional[str] | Omit
            Description of the knowledge base.
        document_column : str | Omit
            Column name for the document column.
        topic_column : str | Omit
            Column name for the topic column.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBase
            The created knowledge base.
        """
        if isinstance(data, str):
            data = Path(data)
        if isinstance(data, list):
            data = ("kb_documents.json", json.dumps(data).encode("utf-8"))

        body = deepcopy_minimal(
            {
                "project_id": project_id,
                "name": name,
                "description": description,
                "document_column": document_column,
                "topic_column": topic_column,
                "file": data,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        response = self._post(
            "/v2/knowledge-bases",
            body=maybe_transform(body, KnowledgeBaseCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBase],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBase:
        """Retrieve a knowledge base by its ID, including its topics.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBase
            The retrieved knowledge base.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = self._get(
            f"/v2/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBase],
        )

        return self._unwrap(response)

    def update(
        self,
        knowledge_base_id: str,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        status: Optional[TaskProgressParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBase:
        """Update an existing knowledge base's metadata.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base to update.
        description : Optional[str] | Omit
            Description of the knowledge base.
        name : Optional[str] | Omit
            Name of the knowledge base.
        project_id : Optional[str] | Omit
            Project ID to update the knowledge base in.
        status : Optional[TaskProgressParam] | Omit
            Task progress status.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBase
            The updated knowledge base.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = self._patch(
            f"/v2/knowledge-bases/{knowledge_base_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "project_id": project_id,
                    "status": status,
                },
                KnowledgeBaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBase],
        )

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[KnowledgeBase]:
        """List all knowledge bases, optionally filtered by project.

        Parameters
        ----------
        project_id : Optional[str] | Omit
            Project ID to list knowledge bases for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[KnowledgeBase]
            A list of knowledge bases.
        """
        response = self._get(
            "/v2/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"project_id": project_id}, KnowledgeBaseListParams),
            ),
            cast_to=APIResponse[List[KnowledgeBase]],
        )

        return self._unwrap(response)

    def delete(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a knowledge base by its ID.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = self._delete(
            f"/v2/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        knowledge_base_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple knowledge bases at once.

        Parameters
        ----------
        knowledge_base_ids : SequenceNotStr[str]
            IDs of the knowledge bases to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"knowledge_base_ids": knowledge_base_ids},
                    KnowledgeBaseBulkDeleteParams,
                ),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    @overload
    def search_documents(
        self,
        knowledge_base_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit = omit,
        filters: Optional[KnowledgeBaseDocumentFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[KnowledgeBaseDocumentRow]: ...

    @overload
    def search_documents(
        self,
        knowledge_base_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit = omit,
        filters: Optional[KnowledgeBaseDocumentFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata]: ...

    def search_documents(
        self,
        knowledge_base_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit = omit,
        filters: Optional[KnowledgeBaseDocumentFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[KnowledgeBaseDocumentRow] | Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata]:
        """Search documents within a knowledge base using filters, sorting, and pagination.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base.
        query : Optional[str] | Omit
            Search query for knowledge base documents.
        order_by : Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit
            Order by criteria for knowledge base documents.
        filters : Optional[KnowledgeBaseDocumentFiltersParam] | Omit
            Search filters.
        limit : int | Omit
            Maximum number of results to return.
        offset : int | Omit
            Number of results to skip.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[KnowledgeBaseDocumentRow] | Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata]
            A list of document rows, or a tuple of document rows and pagination metadata.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = self._post(
            f"/v2/knowledge-bases/{knowledge_base_id}/documents/search",
            body=maybe_transform(
                {"filters": filters, "order_by": order_by, "search": query},
                KnowledgeBaseSearchDocumentsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"limit": limit, "offset": offset},
                    KnowledgeBaseSearchDocumentsParams,
                ),
            ),
            cast_to=APIPaginatedResponse[KnowledgeBaseDocumentRow, None],
        )

        return self._unwrap_paginated(response, include_metadata)

    def retrieve_document(
        self,
        knowledge_base_id: str,
        document_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBaseDocumentDetail:
        """Retrieve detailed information about a specific document in a knowledge base.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base.
        document_id : str
            ID of the document.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBaseDocumentDetail
            The detailed document information.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` or ``document_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        response = self._get(
            f"/v2/knowledge-bases/{knowledge_base_id}/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBaseDocumentDetail],
        )

        return self._unwrap(response)


class AsyncKnowledgeBasesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncKnowledgeBasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncKnowledgeBasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKnowledgeBasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncKnowledgeBasesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        project_id: str,
        data: FileTypes | list[dict[str, Any]] | str,
        description: Optional[str] | Omit = omit,
        document_column: str | Omit = omit,
        topic_column: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBase:
        """Create a new knowledge base by uploading a file of documents.

        Parameters
        ----------
        name : str
            Name of the knowledge base.
        project_id : str
            Project ID to create the knowledge base in.
        data : FileTypes | list[dict[str, Any]] | str
            Data to upload for the knowledge base.
        description : Optional[str] | Omit
            Description of the knowledge base.
        document_column : str | Omit
            Column name for the document column.
        topic_column : str | Omit
            Column name for the topic column.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBase
            The created knowledge base.
        """
        if isinstance(data, str):
            data = Path(data)
        if isinstance(data, list):
            data = ("kb_documents.json", json.dumps(data).encode("utf-8"))

        body = deepcopy_minimal(
            {
                "project_id": project_id,
                "name": name,
                "description": description,
                "document_column": document_column,
                "topic_column": topic_column,
                "file": data,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        response = await self._post(
            "/v2/knowledge-bases",
            body=await async_maybe_transform(body, KnowledgeBaseCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBase],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBase:
        """Retrieve a knowledge base by its ID, including its topics.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBase
            The retrieved knowledge base.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = await self._get(
            f"/v2/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBase],
        )

        return self._unwrap(response)

    async def update(
        self,
        knowledge_base_id: str,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        status: Optional[TaskProgressParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBase:
        """Update an existing knowledge base's metadata.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base to update.
        description : Optional[str] | Omit
            Description of the knowledge base.
        name : Optional[str] | Omit
            Name of the knowledge base.
        project_id : Optional[str] | Omit
            Project ID to update the knowledge base in.
        status : Optional[TaskProgressParam] | Omit
            Task progress status.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBase
            The updated knowledge base.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = await self._patch(
            f"/v2/knowledge-bases/{knowledge_base_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "project_id": project_id,
                    "status": status,
                },
                KnowledgeBaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBase],
        )

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[KnowledgeBase]:
        """List all knowledge bases, optionally filtered by project.

        Parameters
        ----------
        project_id : Optional[str] | Omit
            Project ID to list knowledge bases for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[KnowledgeBase]
            A list of knowledge bases.
        """
        response = await self._get(
            "/v2/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"project_id": project_id}, KnowledgeBaseListParams),
            ),
            cast_to=APIResponse[List[KnowledgeBase]],
        )

        return self._unwrap(response)

    async def delete(
        self,
        knowledge_base_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a knowledge base by its ID.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = await self._delete(
            f"/v2/knowledge-bases/{knowledge_base_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        knowledge_base_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple knowledge bases at once.

        Parameters
        ----------
        knowledge_base_ids : SequenceNotStr[str]
            IDs of the knowledge bases to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/knowledge-bases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"knowledge_base_ids": knowledge_base_ids},
                    KnowledgeBaseBulkDeleteParams,
                ),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    @overload
    async def search_documents(
        self,
        knowledge_base_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit = omit,
        filters: Optional[KnowledgeBaseDocumentFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[KnowledgeBaseDocumentRow]: ...

    @overload
    async def search_documents(
        self,
        knowledge_base_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit = omit,
        filters: Optional[KnowledgeBaseDocumentFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata]: ...

    async def search_documents(
        self,
        knowledge_base_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit = omit,
        filters: Optional[KnowledgeBaseDocumentFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[KnowledgeBaseDocumentRow] | Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata]:
        """Search documents within a knowledge base using filters, sorting, and pagination.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base.
        query : Optional[str] | Omit
            Search query for knowledge base documents.
        order_by : Optional[List[KnowledgeBaseDocumentOrderByParam]] | Omit
            Order by criteria for knowledge base documents.
        filters : Optional[KnowledgeBaseDocumentFiltersParam] | Omit
            Search filters.
        limit : int | Omit
            Maximum number of results to return.
        offset : int | Omit
            Number of results to skip.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[KnowledgeBaseDocumentRow] | Tuple[List[KnowledgeBaseDocumentRow], APIPaginatedMetadata]
            A list of document rows, or a tuple of document rows and pagination metadata.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        response = await self._post(
            f"/v2/knowledge-bases/{knowledge_base_id}/documents/search",
            body=await async_maybe_transform(
                {"filters": filters, "order_by": order_by, "search": query},
                KnowledgeBaseSearchDocumentsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"limit": limit, "offset": offset},
                    KnowledgeBaseSearchDocumentsParams,
                ),
            ),
            cast_to=APIPaginatedResponse[KnowledgeBaseDocumentRow, None],
        )

        return self._unwrap_paginated(response, include_metadata)

    async def retrieve_document(
        self,
        knowledge_base_id: str,
        document_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeBaseDocumentDetail:
        """Retrieve detailed information about a specific document in a knowledge base.

        Parameters
        ----------
        knowledge_base_id : str
            ID of the knowledge base.
        document_id : str
            ID of the document.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        KnowledgeBaseDocumentDetail
            The detailed document information.

        Raises
        ------
        ValueError
            If ``knowledge_base_id`` or ``document_id`` is empty.
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        response = await self._get(
            f"/v2/knowledge-bases/{knowledge_base_id}/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[KnowledgeBaseDocumentDetail],
        )

        return self._unwrap(response)


class KnowledgeBasesResourceWithRawResponse:
    def __init__(self, knowledge_bases: KnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = to_raw_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = to_raw_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.update = to_raw_response_wrapper(
            knowledge_bases.update,
        )
        self.list = to_raw_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = to_raw_response_wrapper(
            knowledge_bases.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            knowledge_bases.bulk_delete,
        )
        self.search_documents = to_raw_response_wrapper(
            knowledge_bases.search_documents,
        )
        self.retrieve_document = to_raw_response_wrapper(
            knowledge_bases.retrieve_document,
        )


class AsyncKnowledgeBasesResourceWithRawResponse:
    def __init__(self, knowledge_bases: AsyncKnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = async_to_raw_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            knowledge_bases.update,
        )
        self.list = async_to_raw_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = async_to_raw_response_wrapper(
            knowledge_bases.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            knowledge_bases.bulk_delete,
        )
        self.search_documents = async_to_raw_response_wrapper(
            knowledge_bases.search_documents,
        )
        self.retrieve_document = async_to_raw_response_wrapper(
            knowledge_bases.retrieve_document,
        )


class KnowledgeBasesResourceWithStreamingResponse:
    def __init__(self, knowledge_bases: KnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = to_streamed_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            knowledge_bases.update,
        )
        self.list = to_streamed_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = to_streamed_response_wrapper(
            knowledge_bases.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            knowledge_bases.bulk_delete,
        )
        self.search_documents = to_streamed_response_wrapper(
            knowledge_bases.search_documents,
        )
        self.retrieve_document = to_streamed_response_wrapper(
            knowledge_bases.retrieve_document,
        )


class AsyncKnowledgeBasesResourceWithStreamingResponse:
    def __init__(self, knowledge_bases: AsyncKnowledgeBasesResource) -> None:
        self._knowledge_bases = knowledge_bases

        self.create = async_to_streamed_response_wrapper(
            knowledge_bases.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            knowledge_bases.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            knowledge_bases.update,
        )
        self.list = async_to_streamed_response_wrapper(
            knowledge_bases.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            knowledge_bases.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            knowledge_bases.bulk_delete,
        )
        self.search_documents = async_to_streamed_response_wrapper(
            knowledge_bases.search_documents,
        )
        self.retrieve_document = async_to_streamed_response_wrapper(
            knowledge_bases.retrieve_document,
        )



"""Project domain types."""

from typing import List, Iterable, Optional, TypedDict
from datetime import datetime
from typing_extensions import Required

from .._types import SequenceNotStr
from .._models import BaseModel
from .evaluation import FailureCategory, FailureCategoryParam

__all__ = [
    "Project",
    "ProjectCreateParams",
    "ProjectUpdateParams",
    "ProjectBulkDeleteParams",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class Project(BaseModel):
    id: str
    created_at: datetime
    description: str
    failure_categories: List[FailureCategory]
    name: str
    updated_at: datetime


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class ProjectCreateParams(TypedDict, total=False):
    name: Required[str]
    description: Optional[str]


class ProjectUpdateParams(TypedDict, total=False):
    description: Optional[str]
    failure_categories: Optional[Iterable[FailureCategoryParam]]
    name: Optional[str]


class ProjectBulkDeleteParams(TypedDict, total=False):
    project_ids: Required[SequenceNotStr[str]]



from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    audit,
    tasks,
    agents,
    checks,
    helpers,
    datasets,
    projects,
    knowledge_bases,
    playground_chats,
    scheduled_evaluations,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, HubClientError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.scans import scans
from .resources.test_cases import test_cases
from .resources.evaluations import evaluations

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "HubClient",
    "AsyncHubClient",
    "Client",
    "AsyncClient",
]

_API_SUFFIX = "/_api"


def _normalize_base_url(base_url: str, *, auto_add_api_suffix: bool) -> str:
    base_url = base_url.rstrip("/")
    if auto_add_api_suffix and not base_url.endswith(_API_SUFFIX):
        base_url += _API_SUFFIX
    return base_url


class HubClient(SyncAPIClient):
    audit_logs: audit.AuditLogsResource
    agents: agents.AgentsResource
    checks: checks.ChecksResource
    datasets: datasets.DatasetsResource
    evaluations: evaluations.EvaluationsResource
    helpers: "helpers.HelpersResource"
    knowledge_bases: knowledge_bases.KnowledgeBasesResource
    playground_chats: playground_chats.PlaygroundChatsResource
    projects: projects.ProjectsResource
    scans: scans.ScansResource
    scheduled_evaluations: scheduled_evaluations.ScheduledEvaluationsResource
    tasks: tasks.TasksResource
    test_cases: test_cases.TestCasesResource
    with_raw_response: HubClientWithRawResponse
    with_streaming_response: HubClientWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        auto_add_api_suffix: bool = True,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        # When debugging, you can set this to False to ignore validation errors.
        _strict_response_validation: bool = True,
    ) -> None:
        """Construct a new synchronous HubClient client instance.

        This automatically infers the `api_key` argument from the `GISKARD_HUB_API_KEY` environment variable if it is not provided.
        The `base_url` will have `/_api` appended automatically unless it is already present or `auto_add_api_suffix=False`.
        """
        if api_key is None:
            api_key = os.environ.get("GISKARD_HUB_API_KEY")
        if api_key is None:
            raise HubClientError(
                "The api_key client option must be set either by passing api_key to the client or by setting the GISKARD_HUB_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("GISKARD_HUB_BASE_URL")
        if base_url is None:
            raise HubClientError(
                "No base_url provided. Pass base_url to the client or set the GISKARD_HUB_BASE_URL environment variable."
            )
        base_url = _normalize_base_url(str(base_url), auto_add_api_suffix=auto_add_api_suffix)

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.audit_logs = audit.AuditLogsResource(self)
        self.agents = agents.AgentsResource(self)
        self.checks = checks.ChecksResource(self)
        self.datasets = datasets.DatasetsResource(self)
        self.evaluations = evaluations.EvaluationsResource(self)
        self.helpers = helpers.HelpersResource(self)
        self.knowledge_bases = knowledge_bases.KnowledgeBasesResource(self)
        self.playground_chats = playground_chats.PlaygroundChatsResource(self)
        self.projects = projects.ProjectsResource(self)
        self.scans = scans.ScansResource(self)
        self.scheduled_evaluations = scheduled_evaluations.ScheduledEvaluationsResource(self)
        self.tasks = tasks.TasksResource(self)
        self.test_cases = test_cases.TestCasesResource(self)
        self.with_raw_response = HubClientWithRawResponse(self)
        self.with_streaming_response = HubClientWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="repeat")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"X-API-Key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Giskard-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        auto_add_api_suffix: bool = True,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            auto_add_api_suffix=auto_add_api_suffix if base_url is not None else False,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncHubClient(AsyncAPIClient):
    audit_logs: audit.AsyncAuditLogsResource
    agents: agents.AsyncAgentsResource
    checks: checks.AsyncChecksResource
    datasets: datasets.AsyncDatasetsResource
    evaluations: evaluations.AsyncEvaluationsResource
    helpers: "helpers.AsyncHelpersResource"
    knowledge_bases: knowledge_bases.AsyncKnowledgeBasesResource
    playground_chats: playground_chats.AsyncPlaygroundChatsResource
    projects: projects.AsyncProjectsResource
    scans: scans.AsyncScansResource
    scheduled_evaluations: scheduled_evaluations.AsyncScheduledEvaluationsResource
    tasks: tasks.AsyncTasksResource
    test_cases: test_cases.AsyncTestCasesResource
    with_raw_response: AsyncHubClientWithRawResponse
    with_streaming_response: AsyncHubClientWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        auto_add_api_suffix: bool = True,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        # When debugging, you can set this to False to ignore validation errors.
        _strict_response_validation: bool = True,
    ) -> None:
        """Construct a new async AsyncHubClient client instance.

        This automatically infers the `api_key` argument from the `GISKARD_HUB_API_KEY` environment variable if it is not provided.
        The `base_url` will have `/_api` appended automatically unless it is already present or `auto_add_api_suffix=False`.
        """
        if api_key is None:
            api_key = os.environ.get("GISKARD_HUB_API_KEY")
        if api_key is None:
            raise HubClientError(
                "The api_key client option must be set either by passing api_key to the client or by setting the GISKARD_HUB_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("GISKARD_HUB_BASE_URL")
        if base_url is None:
            raise HubClientError(
                "No base_url provided. Pass base_url to the client or set the GISKARD_HUB_BASE_URL environment variable."
            )
        base_url = _normalize_base_url(str(base_url), auto_add_api_suffix=auto_add_api_suffix)

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.audit_logs = audit.AsyncAuditLogsResource(self)
        self.agents = agents.AsyncAgentsResource(self)
        self.checks = checks.AsyncChecksResource(self)
        self.datasets = datasets.AsyncDatasetsResource(self)
        self.evaluations = evaluations.AsyncEvaluationsResource(self)
        self.helpers = helpers.AsyncHelpersResource(self)
        self.knowledge_bases = knowledge_bases.AsyncKnowledgeBasesResource(self)
        self.playground_chats = playground_chats.AsyncPlaygroundChatsResource(self)
        self.projects = projects.AsyncProjectsResource(self)
        self.scans = scans.AsyncScansResource(self)
        self.scheduled_evaluations = scheduled_evaluations.AsyncScheduledEvaluationsResource(self)
        self.tasks = tasks.AsyncTasksResource(self)
        self.test_cases = test_cases.AsyncTestCasesResource(self)
        self.with_raw_response = AsyncHubClientWithRawResponse(self)
        self.with_streaming_response = AsyncHubClientWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        # Use repeated query params for arrays, e.g. `include=agent&include=dataset`.
        # This is what FastAPI expects for query params typed as `List[...]`.
        return Querystring(array_format="repeat")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"X-API-Key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Giskard-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        auto_add_api_suffix: bool = True,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            auto_add_api_suffix=auto_add_api_suffix if base_url is not None else False,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class HubClientWithRawResponse:
    def __init__(self, client: HubClient) -> None:
        self.audit_logs = audit.AuditLogsResourceWithRawResponse(client.audit_logs)
        self.agents = agents.AgentsResourceWithRawResponse(client.agents)
        self.checks = checks.ChecksResourceWithRawResponse(client.checks)
        self.datasets = datasets.DatasetsResourceWithRawResponse(client.datasets)
        self.evaluations = evaluations.EvaluationsResourceWithRawResponse(client.evaluations)
        self.knowledge_bases = knowledge_bases.KnowledgeBasesResourceWithRawResponse(client.knowledge_bases)
        self.playground_chats = playground_chats.PlaygroundChatsResourceWithRawResponse(client.playground_chats)
        self.projects = projects.ProjectsResourceWithRawResponse(client.projects)
        self.scans = scans.ScansResourceWithRawResponse(client.scans)
        self.scheduled_evaluations = scheduled_evaluations.ScheduledEvaluationsResourceWithRawResponse(
            client.scheduled_evaluations
        )
        self.tasks = tasks.TasksResourceWithRawResponse(client.tasks)
        self.test_cases = test_cases.TestCasesResourceWithRawResponse(client.test_cases)
        self.helpers = helpers.HelpersResource(client)


class AsyncHubClientWithRawResponse:
    def __init__(self, client: AsyncHubClient) -> None:
        self.audit_logs = audit.AsyncAuditLogsResourceWithRawResponse(client.audit_logs)
        self.agents = agents.AsyncAgentsResourceWithRawResponse(client.agents)
        self.checks = checks.AsyncChecksResourceWithRawResponse(client.checks)
        self.datasets = datasets.AsyncDatasetsResourceWithRawResponse(client.datasets)
        self.evaluations = evaluations.AsyncEvaluationsResourceWithRawResponse(client.evaluations)
        self.knowledge_bases = knowledge_bases.AsyncKnowledgeBasesResourceWithRawResponse(client.knowledge_bases)
        self.playground_chats = playground_chats.AsyncPlaygroundChatsResourceWithRawResponse(client.playground_chats)
        self.projects = projects.AsyncProjectsResourceWithRawResponse(client.projects)
        self.scans = scans.AsyncScansResourceWithRawResponse(client.scans)
        self.scheduled_evaluations = scheduled_evaluations.AsyncScheduledEvaluationsResourceWithRawResponse(
            client.scheduled_evaluations
        )
        self.tasks = tasks.AsyncTasksResourceWithRawResponse(client.tasks)
        self.test_cases = test_cases.AsyncTestCasesResourceWithRawResponse(client.test_cases)
        self.helpers = helpers.AsyncHelpersResource(client)


class HubClientWithStreamedResponse:
    def __init__(self, client: HubClient) -> None:
        self.audit_logs = audit.AuditLogsResourceWithStreamingResponse(client.audit_logs)
        self.agents = agents.AgentsResourceWithStreamingResponse(client.agents)
        self.checks = checks.ChecksResourceWithStreamingResponse(client.checks)
        self.datasets = datasets.DatasetsResourceWithStreamingResponse(client.datasets)
        self.evaluations = evaluations.EvaluationsResourceWithStreamingResponse(client.evaluations)
        self.knowledge_bases = knowledge_bases.KnowledgeBasesResourceWithStreamingResponse(client.knowledge_bases)
        self.playground_chats = playground_chats.PlaygroundChatsResourceWithStreamingResponse(client.playground_chats)
        self.projects = projects.ProjectsResourceWithStreamingResponse(client.projects)
        self.scans = scans.ScansResourceWithStreamingResponse(client.scans)
        self.scheduled_evaluations = scheduled_evaluations.ScheduledEvaluationsResourceWithStreamingResponse(
            client.scheduled_evaluations
        )
        self.tasks = tasks.TasksResourceWithStreamingResponse(client.tasks)
        self.test_cases = test_cases.TestCasesResourceWithStreamingResponse(client.test_cases)
        self.helpers = helpers.HelpersResource(client)


class AsyncHubClientWithStreamedResponse:
    def __init__(self, client: AsyncHubClient) -> None:
        self.audit_logs = audit.AsyncAuditLogsResourceWithStreamingResponse(client.audit_logs)
        self.agents = agents.AsyncAgentsResourceWithStreamingResponse(client.agents)
        self.checks = checks.AsyncChecksResourceWithStreamingResponse(client.checks)
        self.datasets = datasets.AsyncDatasetsResourceWithStreamingResponse(client.datasets)
        self.evaluations = evaluations.AsyncEvaluationsResourceWithStreamingResponse(client.evaluations)
        self.knowledge_bases = knowledge_bases.AsyncKnowledgeBasesResourceWithStreamingResponse(client.knowledge_bases)
        self.playground_chats = playground_chats.AsyncPlaygroundChatsResourceWithStreamingResponse(
            client.playground_chats
        )
        self.projects = projects.AsyncProjectsResourceWithStreamingResponse(client.projects)
        self.scans = scans.AsyncScansResourceWithStreamingResponse(client.scans)
        self.scheduled_evaluations = scheduled_evaluations.AsyncScheduledEvaluationsResourceWithStreamingResponse(
            client.scheduled_evaluations
        )
        self.tasks = tasks.AsyncTasksResourceWithStreamingResponse(client.tasks)
        self.test_cases = test_cases.AsyncTestCasesResourceWithStreamingResponse(client.test_cases)
        self.helpers = helpers.AsyncHelpersResource(client)


Client = HubClient

AsyncClient = AsyncHubClient



"""Task domain types."""

from typing import Union, Optional, TypeAlias, TypedDict
from datetime import datetime
from typing_extensions import Literal, Required

from .scan import ScanProbeAttemptReference
from .user import User
from .._types import SequenceNotStr
from .._models import BaseModel
from .test_case import TestCaseReference
from .evaluation import TestCaseEvaluationReference

__all__ = [
    "Task",
    "TaskStatus",
    "TaskPriority",
    "TaskListParams",
    "TaskCreateParams",
    "TaskUpdateParams",
    "TaskBulkDeleteParams",
]


# ---------------------------------------------------------------------------
# Enums
# ---------------------------------------------------------------------------

TaskStatus: TypeAlias = Literal["open", "in_progress", "resolved"]
TaskPriority: TypeAlias = Literal["low", "medium", "high"]


# ---------------------------------------------------------------------------
# Task model
# ---------------------------------------------------------------------------


class Task(BaseModel):
    id: str
    project_id: str
    priority: Optional[TaskPriority] = None
    status: TaskStatus
    description: str
    created_by: User
    assignees: SequenceNotStr[User]
    references: SequenceNotStr[Union[TestCaseEvaluationReference, ScanProbeAttemptReference, TestCaseReference]]
    created_at: datetime
    updated_at: datetime


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class TaskListParams(TypedDict, total=False):
    project_id: Required[str]


class TaskCreateParams(TypedDict, total=False):
    project_id: Required[str]
    priority: Optional[TaskPriority]
    status: TaskStatus
    description: Required[str]
    assignee_ids: SequenceNotStr[str]
    evaluation_result_id: Optional[str]
    dataset_test_case_id: Optional[str]
    probe_attempt_id: Optional[str]
    disable_test: bool
    hide_result: bool


class TaskUpdateParams(TypedDict, total=False):
    assignee_ids: Optional[SequenceNotStr[str]]
    description: Optional[str]
    priority: Optional[TaskPriority]
    status: Optional[TaskStatus]
    set_test_case_status: Optional[str]


class TaskBulkDeleteParams(TypedDict, total=False):
    task_ids: Required[SequenceNotStr[str]]
